#ifndef CpuLoad_h
#define CpuLoad_h

// $Id: CpuLoad.h,v 1.21 2014/09/12 08:43:12 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"

#include <QTextStream>
#include <QDateTime>
#include <QHash>
#include <QList>
#include <QSet>


namespace Cpu
{

    //! measures relative cpu load from /proc/stat file
    class CpuLoad: public Counter
    {

        public:

        //! Data type enumeration
        enum DataType
        {
            User = 1<<0,
            System = 1<<1,
            Nice = 1<<2,
            All = User|System|Nice
        };

        Q_DECLARE_FLAGS( DataTypes, DataType );

        //! constructor
        CpuLoad( void );

        //! cpu load data
        class Data
        {

            public:

            //! constructor
            Data( void ):
                user_( 0 ),
                nice_( 0 ),
                system_( 0 ),
                total_( 0 ),
                time_( 0 ),
                valid_( false )
            {}

            //! difference between two points
            Data operator - ( const Data& data ) const
            {
                Data out( *this );
                out.user_ = user_ - data.user_;
                out.nice_ = nice_ - data.nice_;
                out.system_ = system_ - data.system_;
                out.total_ = total_ - data.total_;
                out.time_ = time_ - data.time_;
                out.valid_ = ( valid_ && data.valid_ && out.time_ > 0 );
                return out;
            }

            //! load from stream
            friend QTextStream &operator >> ( QTextStream &in, Data& data )
            {
                in >> data.user_ >> data.nice_ >> data.system_;

                // idle is supposed to be the last number
                int tmp = 0;
                data.total_ = 0;
                while( in.status() == QTextStream::Ok )
                {
                    data.total_ += tmp;
                    in >> tmp;
                }

                data.valid_ = data.total_ > 0;
                data.total_ += data.user_ + data.nice_ + data.system_;

                return in;
            }

            //! user time
            int user( void ) const
            { return user_; }

            //! nice time
            int nice( void ) const
            { return nice_; }

            //! system time
            int system( void ) const
            { return system_; }

            //! sum
            int sum( DataTypes type = All ) const
            {
                int out( 0 );
                if( type & User ) out += user();
                if( type & System ) out += system();
                if( type & Nice ) out += nice();
                return out;
            }

            //! idle time
            int total( void ) const
            { return total_; }

            //! total_cpu over total (in %)
            int ratio( DataTypes type = All ) const
            { return 100*sum(type)/total_; }

            //! time
            void setTime( int time )
            { time_ = time; }

            //! time
            int time( void ) const
            { return time_; }

            //! validity
            bool valid( void ) const
            { return valid_; }

            //! used to sort data according to selected data type
            class LessFTor
            {

                public:

                //! constructor
                LessFTor( DataTypes type ):
                    type_( type )
                {}

                //! predicate
                bool operator()( const Data& first, const Data& second ) const
                { return double(first.sum( type_ ))/first.time() < double( second.sum( type_ ))/second.time(); }

                private:

                //! data type
                DataTypes type_;

            };

            private:

            //! user time
            int user_;

            //! nice time
            int nice_;

            //! system time
            int system_;

            //! idle time
            int total_;

            //! measurement time
            int time_;

            //! true if current data are valid
            bool valid_;

        };

        //! shortcut for data list
        using DataList = QList< Data >;

        //! CpuLoad data samples storage
        class Samples: public Counter
        {

            public:

            //! construtor
            Samples( int sampleSize = 0, int dataSize = 0 ):
                Counter( "CpuLoad::Samples" ),
                sampleSize_( sampleSize ),
                dataSize_( dataSize )
            { }

            //! destructor
            ~Samples( void )
            {}

            //! set sample size
            void setSampleSize( int );

            //! set data point size
            void setDataSize( int );

            //! retrieve data
            const DataList& data( void ) const
            { return dataList_; }

            //! add a data point
            void add( const Data& data );

            private:

            //! list of cpu load measurement
            DataList sampleList_;

            //! list of cpu load differences
            DataList dataList_;

            //! number of samples to be averaged to make the data points
            int sampleSize_;

            //! number of data point to store
            int dataSize_;

        };

        //! shortcut for Samples set
        using SampleMap = QHash< QString, Samples >;

        //! cpu names
        using CpuNames = QSet<QString>;

        //! get available cpu names
        CpuNames cpuNames( void ) const
        {
            CpuNames out;
            for( SampleMap::const_iterator iter = sampleMap_.begin(); iter != sampleMap_.end(); iter++ )
            { out.insert( iter.key() ); }
            return out;
        }

        //! changes sampleSize_
        void setSampleSize( int value )
        {
            sampleSize_ = value;
            for( SampleMap::iterator iter = sampleMap_.begin(); iter != sampleMap_.end(); iter++ )
            { iter.value().setSampleSize( value ); }
        }

        //! changes sampleSize_
        void setDataSize( int value )
        {
            dataSize_ = value;
            for( SampleMap::iterator iter = sampleMap_.begin(); iter != sampleMap_.end(); iter++ )
            { iter.value().setDataSize( value ); }
        }

        //! retrieve data for a given cpu name
        DataList data( const QString& cpuName = "cpu" ) const;

        //! update all Samples from file
        void update( void );

        //! used to sort data according to selected data type
        class LessFTor
        {

            public:

            //! constructor
            LessFTor( DataTypes type ):
                type_( type )
            {}

            //! predicate
            bool operator()( const Data& first, const Data& second ) const
            { return double(first.sum( type_ ))/first.time() < double( second.sum( type_ ))/second.time(); }

            private:

            //! data type
            DataTypes type_;

        };

        private:

        //! filename
        const QString filename_;

        //! number of samples to average on
        int sampleSize_;

        //! number of data to be kept for each cpu
        int dataSize_;

        //! data samples
        SampleMap sampleMap_;

        //! timer
        QTime timer_;

    };

};

Q_DECLARE_OPERATORS_FOR_FLAGS( Cpu::CpuLoad::DataTypes )

#endif
