// $Id: ApplicationDataModel.cpp,v 1.15 2014/02/09 18:22:57 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ApplicationDataModel.h"
#include "ApplicationDataModel.moc"
#include "Debug.h"

#include <QApplication>

//_______________________________________________
const QString ApplicationDataModel::columnTitles_[ ApplicationDataModel::nColumns ] =
{
    tr( "Name" ),
    tr( "Position" ),
    tr( "Size" )
};

//_______________________________________________________________
ApplicationDataModel::ApplicationDataModel( QObject* parent ):
    ListModel<ApplicationData>( parent ),
    Counter( "ApplicationDataModel" )
{

    Debug::Throw( "ApplicationDataModel::ApplicationDataModel.\n" );

}

//__________________________________________________________________
Qt::ItemFlags ApplicationDataModel::flags(const QModelIndex &index) const
{
    if( !index.isValid() ) return Qt::ItemIsDropEnabled;
    return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled;
}

//__________________________________________________________________
QVariant ApplicationDataModel::data( const QModelIndex& index, int role ) const
{
    Debug::Throw( "ApplicationDataModel::data.\n" );

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated command
    const ApplicationData& applicationData( get()[index.row()] );

    // return text associated to command and column
    switch( role )
    {

        case Qt::DisplayRole:
        {
            switch( index.column() )
            {
                case Name: return applicationData.name();
                case Position:
                {
                    QString buffer;
                    QTextStream( &buffer ) << applicationData.geometry().topLeft().x()  << "x" << applicationData.geometry().topLeft().y();
                    return buffer;
                }
                case Size:
                {
                    QString buffer;
                    QTextStream( &buffer ) << applicationData.geometry().width()  << "x" << applicationData.geometry().height();
                    return buffer;
                }
                default: return QVariant();
            }
        }

        case Qt::DecorationRole:
        {
            if( index.column() == Name ) return applicationData.icon();
            break;
        }

        default: break;

    }

    return QVariant();

}

//__________________________________________________________________
QVariant ApplicationDataModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//______________________________________________________________________
QStringList ApplicationDataModel::mimeTypes( void ) const
{
    QStringList types;
    types << ApplicationData::MimeType << "text/plain";
    return types;
}



//______________________________________________________________________
QMimeData* ApplicationDataModel::mimeData(const QModelIndexList &indexes) const
{

    // create mime data
    QMimeData *mime = new QMimeData();

    // add keywords mimetype
    foreach( const QModelIndex& index, indexes )
    { if( index.isValid() && index.column() == Name ) mime->setData( ApplicationData::MimeType, QString::number(index.row()).toLatin1() ); }

    // add plain text mimetype
    QString buffer;
    foreach( const QModelIndex& index, indexes )
    {
        if( !( index.isValid() && index.column() == Name ) ) continue;
        const ApplicationData& data( get( index ) );
        buffer += data.name()
            + QString( tr( " (%1,%2) (%3x%4 ) \n" ) )
            .arg(data.geometry().x() )
            .arg(data.geometry().y() )
            .arg(data.geometry().width() )
            .arg(data.geometry().height() );
    }

    // set plain text data
    mime->setData( "text/plain", buffer.toLatin1() );

    return mime;

}

//__________________________________________________________________
bool ApplicationDataModel::dropMimeData(const QMimeData* data , Qt::DropAction action, int row, int column, const QModelIndex& parent)
{

    Debug::Throw( "ApplicationDataModel::dropMimeData\n" );

    // check action
    if( action == Qt::IgnoreAction) return true;
    if( row < 0 ) return false;
    if( data->hasFormat( ApplicationData::MimeType ) )
    {

        // retrieve row of dragged item and send signal
        const int oldRow( data->data( ApplicationData::MimeType ).toInt() );
        emit itemOrderChanged( oldRow, row );
        return true;

    }

    return false;
}
