#ifndef MemoryLoad_h
#define MemoryLoad_h

// $Id: MemoryLoad.h,v 1.19 2014/09/12 08:43:12 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "KernelVersion.h"
#include "Counter.h"

#include <QTextStream>
#include <QList>

namespace Memory
{

    //! measures relative memory load from /proc/stat file
    class MemoryLoad: public Counter
    {

        public:

        //! Data type enumeration
        enum DataType
        {
            Memory = 1<<0,
            Swap = 1<<1,
            All = Memory|Swap
        };

        Q_DECLARE_FLAGS( DataTypes, DataType )

        //! constructor
        MemoryLoad( void );

        //! cpu load data
        class Data
        {

            public:

            //! constructor
            Data( void ):
                mem_( 0 ),
                swap_( 0 ),
                total_mem_( 0 ),
                total_swap_( 0 ),
                valid_( false )
            {}

            //! load from stream
            friend QTextStream &operator << ( QTextStream &out, const Data& data )
            {
                out << data.mem_
                    << " " << data.total_mem_
                    << " " << data.swap_
                    << " " << data.total_swap_
                    << " " << ((data.valid_) ? "valid":"invalid" );
                return out;
            }

            //! sum points
            Data& operator += ( const Data& data )
            {
                mem_ += data.mem_;
                swap_ +=  data.swap_;
                    total_mem_ +=  data.total_mem_;
                    total_swap_ += data.total_swap_;
                    valid_ &= data.valid_;
                return *this;
            }

            //! subtract points
            Data operator - ( const Data& data ) const
            {
                Data out( *this );
                out.mem_ -= data.mem_;
                out.swap_ -=  data.swap_;
                out.total_mem_ -=  data.total_mem_;
                out.total_swap_ -= data.total_swap_;
                out.valid_ &= data.valid_;
                return out;
            }

            //! divide point by an integer
            Data divide(const unsigned long& count ) const
            {
                Q_ASSERT( count > 0 );
                Data out( *this );
                out.mem_ /= count;
                out.swap_ /= count;
                out.total_mem_ /= count;
                out.total_swap_ /= count;
                return out;
            }

            //! memory used
            const unsigned long& memory( void ) const
            { return mem_; }

            //! memory used
            unsigned long& memory( void )
            { return mem_; }

            //! total memory
            const unsigned long& totalMemory( void ) const
            { return total_mem_; }

            //! total memory
            unsigned long& totalMemory( void )
            { return total_mem_; }

            //! swap used
            const unsigned long& swap( void ) const
            { return swap_; }

            //! swap used
            unsigned long& swap( void )
            { return swap_; }

            //! total swap
            const unsigned long& totalSwap( void ) const
            { return total_swap_; }

            //! total swap
            unsigned long& totalSwap( void )
            { return total_swap_; }

            //! get mem load ratio (in %)
            int memoryRatio( void ) const
            { return (total_mem_ ) ? static_cast<int>((100*mem_)/total_mem_):0; }

            //! get mem load ratio (in %)
            int swapRatio( void ) const
            { return (total_swap_ ) ? static_cast<int>((100*swap_)/total_swap_):0; }

            //! sum
            unsigned long sum( const unsigned int& type ) const
            {
                unsigned long out(0);
                if( type & Memory ) out += memory();
                if( type & Swap ) out += swap();
                return out;
            }

            //! total
            unsigned long total( const unsigned int& type ) const
            {
                unsigned long out(0);
                if( type & Memory ) out += totalMemory();
                if( type & Swap ) out += totalSwap();
                return out;
            }

            //! get validity
            bool valid( void ) const
            { return valid_; }

            //! get validity
            bool& valid( void )
            { return valid_; }

            private:

            //! memory
            unsigned long mem_;

            //! swap
            unsigned long swap_;

            //! total memory
            unsigned long total_mem_;

            //! total swap
            unsigned long total_swap_;

            //! true if current data are valid
            bool valid_;

        };

        //! shortcut to access data list
        using DataList = QList< Data >;

        //! kernel version
        void setKernelVersion( Main::KernelVersion version )
        { version_ = version; }

        //! sample size
        void setSampleSize( int value )
        {
            sampleSize_ = std::max<size_t>( value, 1 );
            while( sampleList_.size() > sampleSize_ )
            { sampleList_.removeFirst(); }
        }

        //! changes data size
        void setDataSize( int value )
        {
            dataSize_ = std::max<size_t>( value, 1 );
            while( dataList_.size() > dataSize_ )
            { dataList_.removeFirst(); }
        }

        //! retrieve differencies
        DataList data() const
        { return dataList_; }

        //! update last value and differences
        void update();

        //! used to sort data according to selected data type
        class LessFTor
        {

            public:

            //! constructor
            LessFTor( const unsigned int& type ):
                type_( type )
            {}

            //! predicate
            bool operator()( const Data& first, const Data& second ) const
            { return first.sum( type_ ) < second.sum( type_ ); }

            private:

            //! data type
            unsigned int type_;

        };

        private:

        //! load current data from file
        Data _data( void );

        //! Kernel version
        Main::KernelVersion version_;

        //! filename
        const QString filename_;

        //! number of samples to average on
        int sampleSize_;

        //! number of data to keep in memory
        int dataSize_;

        //! last loaded data
        DataList sampleList_;

        //! averaged data points
        DataList dataList_;

    };
};
#endif
