// $Id: EditPictureDialog.cpp,v 1.21 2014/09/26 08:06:19 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "EditPictureDialog.h"
#include "EditPictureDialog.moc"

#include "AnimatedLineEditor.h"
#include "Debug.h"
#include "GridLayout.h"
#include "ImageFileDialog.h"
#include "InformationDialog.h"
#include "XmlOptions.h"

#include <QLayout>

namespace Pictures
{

    //_______________________________________________________
    void PictureEditor::_browse( void )
    {

        Debug::Throw( "PictureEditor::_browse" );
        ImageFileDialog dialog( this );

        // propagate modes
        dialog.setAcceptMode( _acceptMode() );
        dialog.setFileMode( _fileMode() );

        // select current file
        File current( editor().text() );
        if( current.isDirectory() || ( current = current.path() ).isDirectory() )
            dialog.setDirectory( current );

        if( dialog.exec() != QDialog::Accepted ) return;

        // retrieve selected files
        QStringList files( dialog.selectedFiles() );

        // check file size
        if( files.size() > 1 )
        {

            InformationDialog( this, tr( "Too many files selected." ) ).exec();
            return;

        } else if( files.size() < 1 ) {

            InformationDialog( this, tr( "No file selected." ) ).exec();
            return;

        } else setFile( files.front() );

    }

    //_______________________________________________________
    EditPictureDialog::EditPictureDialog( QWidget* parent ):
        CustomDialog( parent )
    {

        Debug::Throw( "EditPictureDialog::EditPictureDialog.\n" );
        setWindowTitle( tr( "Picture Configuration - Picture Viewer" ) );
        setOptionName( "EDIT_PICTURE_DIALOG" );

        QHBoxLayout* hLayout( new QHBoxLayout() );
        hLayout->setMargin(0);
        hLayout->setSpacing(10);
        mainLayout().addLayout( hLayout );

        iconLabel_ = new QLabel( this );
        iconLabel_->setMargin(10);
        hLayout->addWidget( iconLabel_, 0, Qt::AlignTop );

        QVBoxLayout *vLayout( new QVBoxLayout() );
        vLayout->setMargin(0);
        vLayout->setSpacing(2);
        hLayout->addLayout( vLayout );

        vLayout->addWidget( editor_ = new PictureEditor( this ) );

        GridLayout* gridLayout = new GridLayout();
        gridLayout->setMargin(0);
        gridLayout->setSpacing(2);
        gridLayout->setMaxCount(2);

        gridLayout->setColumnStretch( 1, 1 );

        vLayout->addLayout( gridLayout );

        QLabel* label;
        gridLayout->addWidget( label = new QLabel( tr( "Width:" ), this ) );
        gridLayout->addWidget( widthSpinBox_ = new QSpinBox( this ) );
        widthSpinBox_->setMinimum(0);
        widthSpinBox_->setMaximum(2024);
        label->setBuddy( widthSpinBox_ );

        gridLayout->addWidget( label = new QLabel( tr( "Height:" ), this ) );
        gridLayout->addWidget( heightSpinBox_ = new QSpinBox( this ) );
        heightSpinBox_->setMinimum(0);
        heightSpinBox_->setMaximum(2024);
        label->setBuddy( heightSpinBox_ );

        vLayout->addWidget( visibilityCheckBox_ = new QCheckBox( tr( "Visible" ), this ) );

        connect( &editor_->editor(), SIGNAL(textChanged(QString)), SLOT(_updateIcon()) );

        adjustSize();
        return;

    }

    //_________________________________________________
    void EditPictureDialog::setAttributes( const PictureAttributes& attributes )
    {

        Debug::Throw( "EditPictureDialog::setPictureAttributes.\n" );
        attributes_ = attributes;
        editor_->editor().setText( attributes_.filename() );
        widthSpinBox_->setValue( attributes_.size().width() );
        heightSpinBox_->setValue( attributes_.size().height() );
        visibilityCheckBox_->setChecked( attributes_.isVisible() );

        return;
    }

    //___________________________________________________
    PictureAttributes EditPictureDialog::attributes( void ) const
    {

        PictureAttributes attributes( attributes_ );
        attributes.setFilename( editor_->editor().text() );
        attributes.setSize( QSize( widthSpinBox_->value(), heightSpinBox_->value() ) );
        attributes.setVisible( visibilityCheckBox_->isChecked() );
        return attributes;

    }

    //___________________________________________________
    void EditPictureDialog::_updateIcon( void )
    {

        static const QSize scaled_size( 64, 64 );

        // icon
        QPixmap icon;
        QString filename( editor_->editor().text() );
        if( filename != PictureAttributes::NO_PICTURE && QFileInfo( filename ).exists() )
        { icon = QPixmap( filename ); }

        if( icon.isNull() ) iconLabel_->setPixmap( QPixmap() );

        // scale
        icon = icon.scaled( scaled_size, Qt::KeepAspectRatio, Qt::SmoothTransformation );

        // assign to label
        iconLabel_->setPixmap( icon );

    }
}
