// $Id: Temperature.cpp,v 1.25 2014/02/10 10:07:45 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/
#include "Temperature.h"
#include "Temperature.moc"

#include "CustomProcess.h"
#include "Debug.h"
#include "IOString.h"
#include "XmlOptions.h"

#include <QFile>
#include <QStringList>

namespace Temperature
{

    //________________________________________________
    Temperature::Temperature( void ):
        Counter( "Temperature" ),
        useSensors_( true ),
        sampleSize_( 1 ),
        dataSize_( 10 )
    {
        connect( &process_, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(_commandCompleted(int,QProcess::ExitStatus)) );
        connect( &process_, SIGNAL(readyReadStandardOutput()), SLOT(_readStdout()) );
        connect( &process_, SIGNAL(readyReadStandardError()), SLOT(_readStderr()) );

    }

    //________________________________________________
    void Temperature::update( void )
    {

        if( useSensors() )
        {

            // do nothing if process already running
            if( process_.state() == QProcess::NotRunning )
            { process_.start( QStringList() << XmlOptions::get().raw("LM_SENSORS" ) << "-u" ); }

            return;

        } else {

            // open file
            QFile in( XmlOptions::get().raw( "ACPI_FILE" ) );
            if( !in.open( QIODevice::ReadOnly ) )
            {
                Debug::Throw( "Temperature::update - cannot open file.\n" );
                return;
            }

            QStringList lines( QString( in.readAll() ).split( "\n" ) );
            for( QStringList::iterator iter = lines.begin(); iter != lines.end(); ++iter )
            {

                QString line( *iter );
                QString tag = "temperature:";
                int begin( line.indexOf( tag ) );
                if( begin < 0 ) continue;
                begin += tag.size();

                Data data;
                line = line.mid( begin );
                QTextStream stream( &line );
                stream >> data;

                QString probeName( "" );

                // initialize Samples if new
                if( sampleMap_.find( probeName ) == sampleMap_.end() )
                {
                    sampleMap_[ probeName ].setSampleSize( sampleSize_ );
                    sampleMap_[ probeName ].setDataSize( dataSize_ );
                }

                // add current data to matching sample
                sampleMap_[ probeName ].add( data );

            }

        }

        return;

    }

    //________________________________________________
    Temperature::DataList Temperature::data( const QString& probeName ) const
    {
        Debug::Throw() << "Temperature::data - probName: " << probeName << endl;
        SampleMap::const_iterator iter( sampleMap_.find( probeName ) );
        if( iter == sampleMap_.end() ) return DataList();
        else return iter.value().data();
    }

    //______________________________________________________________
    void Temperature::_commandCompleted( int return_code, QProcess::ExitStatus status )
    {

        Debug::Throw() << "Temperature::_commandCompleted" << endl;

        // check status
        if( status != QProcess::NormalExit || process_.error() == QProcess::FailedToStart )
        { return; }

        // matching regexp
        QRegExp regexp(
            "(\\b(?:\\S+)\\b(?:\\s\\d)?(?:\\sTemperature)?):"
            "\\s*temp\\d_input:\\s*(\\d+\\.\\d+)" );

        int position(0);
        while( ( position = regexp.indexIn( stdoutBuffer_.text(), position ) ) >= 0 )
        {
            QStringList captures( regexp.capturedTexts() );
            if( captures.size() < 3 ) continue;
            const QString probeName( captures[1] );

            // initialize Samples if new
            if( sampleMap_.find( probeName ) == sampleMap_.end() )
            {
                Debug::Throw() << "Temperature::_commandCompleted - adding probe: " << probeName << endl;
                sampleMap_[ probeName ].setSampleSize( sampleSize_ );
                sampleMap_[ probeName ].setDataSize( dataSize_ );
            }

            // add current data to matching sample
            sampleMap_[ probeName ].add( Data( captures[2].toDouble(), true ) );

            // increment position
            position += regexp.matchedLength();
        }

        // clear buffer
        stderrBuffer_.clear();
        stdoutBuffer_.clear();

        return;

    }

    //___________________________________________________________________
    void Temperature::_readStdout(  void )
    {
        process_.setReadChannel( QProcess::StandardOutput );
        IOString buffer( process_ );
        stdoutBuffer_.append( buffer );
    }

    //___________________________________________________________________
    void Temperature::_readStderr(  void )
    {
        process_.setReadChannel( QProcess::StandardError );
        IOString buffer( process_ );
        stderrBuffer_.append( buffer );
    }

    //________________________________________________
    void Temperature::Samples::setSampleSize( int sampleSize )
    {
        sampleSize_ = sampleSize;
        while( sampleList_.size() > sampleSize_ )
        { sampleList_.removeFirst(); }
    }

    //________________________________________________
    void Temperature::Samples::setDataSize( int dataSize )
    {
        dataSize_ = dataSize;
        while( dataList_.size() > dataSize_ )
        { dataList_.removeFirst(); }
    }

    //________________________________________________
    void Temperature::Samples::add( const Temperature::Data& data )
    {

        // add data to samples
        sampleList_ << data;
        if( sampleList_.size() > sampleSize_ )
        {

            sampleList_.removeFirst();

            Data sum;
            for( DataList::iterator iter = sampleList_.begin(); iter != sampleList_.end(); iter ++ )
                if( ! sum.valid() ) sum = (*iter);
            else sum = sum + (*iter);
            dataList_ << sum.divide( sampleSize_ );

            if( dataList_.size() > dataSize_ ) dataList_.removeFirst();

        }

        return;

    }



}
